@@ -422,10 +422,15 @@ QColor Element::curColor(const Element* proxy) const
             marked = note->mark();
             }
       if (proxy->selected() || marked ) {
+            QColor originalColor;
             if (track() == -1)
-                  return MScore::selectColor[0];
+                  originalColor = MScore::selectColor[0];
             else
-                  return MScore::selectColor[voice()];
+                  originalColor = MScore::selectColor[voice()];
+            if (proxy->visible())
+                  return originalColor;
+            else
+                  return originalColor.lighter(200);
             }
       if (!proxy->visible())
             return Qt::gray;
@@ -1505,8 +1510,8 @@ bool Element::setProperty(P_ID propertyId, const QVariant& v)
                   _placement = Placement(v.toInt());
                   break;
             default:
-                  qFatal("Element::setProperty: unknown <%s>(%hhd), data <%s>",
-                     propertyName(propertyId), propertyId, qPrintable(v.toString()));
+                  qFatal("Element::setProperty: unknown <%s>(%d), data <%s>",
+                     propertyName(propertyId), int(propertyId), qPrintable(v.toString()));
                   return false;
             }
       setGenerated(false);
