@@ -114,14 +114,19 @@ void Preferences::init()
 
       rememberLastConnections = true;
 
-      alsaDevice         = "default";
-      alsaSampleRate     = 48000;
-      alsaPeriodSize     = 1024;
-      alsaFragments      = 3;
-      portaudioDevice    = -1;
-      portMidiInput      = "";
+      alsaDevice                = "default";
+      alsaSampleRate            = 48000;
+      alsaPeriodSize            = 1024;
+      alsaFragments             = 3;
+      portaudioDevice           = -1;
+      portMidiInput             = "";
+      portMidiInputBufferCount  = 100;
+      portMidiOutput            = "";
+      portMidiOutputBufferCount = 65536;
+      portMidiOutputLatencyMilliseconds = 0;
 
       antialiasedDrawing       = true;
+      limitScrollArea          = false;
       sessionStart             = SessionStart::SCORE;
       startScore               = ":/data/My_First_Score.mscz";
       defaultStyleFile         = "";
@@ -153,10 +158,12 @@ void Preferences::init()
 
       mag                     = 1.0;
 
-#if defined(Q_OS_MAC) || defined(Q_OS_WIN)
+#if defined(Q_OS_MAC) || (defined(Q_OS_WIN) && !defined(FOR_WINSTORE))
       checkUpdateStartup      = true;
+      checkExtensionsUpdateStartup = true;
 #else
       checkUpdateStartup      = false;
+      checkExtensionsUpdateStartup = false;
 #endif
 
       followSong              = true;
@@ -185,6 +192,7 @@ void Preferences::init()
       myTemplatesPath = QFileInfo(QString("%1/%2").arg(wd).arg(QCoreApplication::translate("templates_directory",  "Templates"))).absoluteFilePath();
       myPluginsPath   = QFileInfo(QString("%1/%2").arg(wd).arg(QCoreApplication::translate("plugins_directory",    "Plugins"))).absoluteFilePath();
       mySoundfontsPath = QFileInfo(QString("%1/%2").arg(wd).arg(QCoreApplication::translate("soundfonts_directory", "Soundfonts"))).absoluteFilePath();
+      myExtensionsPath = QFileInfo(QString("%1/%2").arg(wd).arg(QCoreApplication::translate("extensions_directory", "Extensions"))).absoluteFilePath();
 
       MScore::setNudgeStep(.1);         // cursor key (default 0.1)
       MScore::setNudgeStep10(1.0);      // Ctrl + cursor key (default 1.0)
@@ -257,11 +265,14 @@ void Preferences::write()
       s.setValue("alsaPeriodSize",     alsaPeriodSize);
       s.setValue("alsaFragments",      alsaFragments);
       s.setValue("portaudioDevice",    portaudioDevice);
-      s.setValue("portMidiInput",   portMidiInput);
+      s.setValue("portMidiInput",      portMidiInput);
+      s.setValue("portMidiOutput",     portMidiOutput);
+      s.setValue("portMidiOutputLatencyMilliseconds", portMidiOutputLatencyMilliseconds);
 
       s.setValue("layoutBreakColor",   MScore::layoutBreakColor.name(QColor::NameFormat::HexArgb));
       s.setValue("frameMarginColor",   MScore::frameMarginColor.name(QColor::NameFormat::HexArgb));
       s.setValue("antialiasedDrawing", antialiasedDrawing);
+      s.setValue("limitScrollArea",    limitScrollArea);
       switch(sessionStart) {
             case SessionStart::EMPTY:  s.setValue("sessionStart", "empty"); break;
             case SessionStart::LAST:   s.setValue("sessionStart", "last"); break;
@@ -323,6 +334,7 @@ void Preferences::write()
       s.setValue("myTemplatesPath", myTemplatesPath);
       s.setValue("myPluginsPath", myPluginsPath);
       s.setValue("mySoundfontsPath", mySoundfontsPath);
+      s.setValue("myExtensionsPath", myExtensionsPath);
       s.remove("sfPath");
 
       s.setValue("hraster", MScore::hRaster());
@@ -337,6 +349,7 @@ void Preferences::write()
 
       //update
       s.setValue("checkUpdateStartup", checkUpdateStartup);
+      s.setValue("checkExtensionsUpdateStartup", checkExtensionsUpdateStartup);
 
       s.setValue("useMidiRemote", useMidiRemote);
       for (int i = 0; i < MIDI_REMOTES; ++i) {
@@ -425,9 +438,12 @@ void Preferences::read()
       alsaFragments      = s.value("alsaFragments", alsaFragments).toInt();
       portaudioDevice    = s.value("portaudioDevice", portaudioDevice).toInt();
       portMidiInput      = s.value("portMidiInput", portMidiInput).toString();
+      portMidiOutput     = s.value("portMidiOutput", portMidiOutput).toString();
+      portMidiOutputLatencyMilliseconds = s.value("portMidiOutputLatencyMilliseconds", portMidiOutputLatencyMilliseconds).toInt();
       MScore::layoutBreakColor   = readColor("layoutBreakColor", MScore::layoutBreakColor);
       MScore::frameMarginColor   = readColor("frameMarginColor", MScore::frameMarginColor);
       antialiasedDrawing      = s.value("antialiasedDrawing", antialiasedDrawing).toBool();
+      limitScrollArea         = s.value("limitScrollArea", limitScrollArea).toBool();
 
       defaultStyleFile         = s.value("defaultStyle", defaultStyleFile).toString();
 
@@ -486,6 +502,7 @@ void Preferences::read()
       pl.removeAll(QFileInfo(QString("%1%2").arg(mscoreGlobalShare).arg("sound")).absoluteFilePath());
       mySoundfontsPath = pl.join(";");
       mySoundfontsPath = s.value("mySoundfontsPath", mySoundfontsPath).toString();
+      myExtensionsPath = s.value("myExtensionsPath",    myExtensionsPath).toString();
 
       //Create directories if they are missing
       QDir dir;
@@ -496,6 +513,7 @@ void Preferences::read()
       dir.mkpath(myPluginsPath);
       foreach (QString path, mySoundfontsPath.split(";"))
             dir.mkpath(path);
+      dir.mkpath(myExtensionsPath);
 
       MScore::setHRaster(s.value("hraster", MScore::hRaster()).toInt());
       MScore::setVRaster(s.value("vraster", MScore::vRaster()).toInt());
@@ -509,6 +527,7 @@ void Preferences::read()
       MScore::setVerticalOrientation(s.value("verticalPageOrientation", MScore::verticalOrientation()).toBool());
 
       checkUpdateStartup = s.value("checkUpdateStartup", checkUpdateStartup).toBool();
+      checkExtensionsUpdateStartup = s.value("checkExtensionsUpdateStartup", checkExtensionsUpdateStartup).toBool();
 
       QString ss(s.value("sessionStart", "score").toString());
       if (ss == "last")
@@ -585,6 +604,7 @@ PreferenceDialog::PreferenceDialog(QWidget* parent)
       myPluginsButton->setIcon(*icons[int(Icons::fileOpen_ICON)]);
       mySoundfontsButton->setIcon(*icons[int(Icons::edit_ICON)]);
       myImagesButton->setIcon(*icons[int(Icons::fileOpen_ICON)]);
+      myExtensionsButton->setIcon(*icons[int(Icons::fileOpen_ICON)]);
 
       bgWallpaperSelect->setIcon(*icons[int(Icons::fileOpen_ICON)]);
       fgWallpaperSelect->setIcon(*icons[int(Icons::fileOpen_ICON)]);
@@ -679,6 +699,7 @@ PreferenceDialog::PreferenceDialog(QWidget* parent)
       connect(myPluginsButton, SIGNAL(clicked()), SLOT(selectPluginsDirectory()));
       connect(myImagesButton, SIGNAL(clicked()), SLOT(selectImagesDirectory()));
       connect(mySoundfontsButton, SIGNAL(clicked()), SLOT(changeSoundfontPaths()));
+      connect(myExtensionsButton, SIGNAL(clicked()), SLOT(selectExtensionsDirectory()));
 
       connect(updateTranslation, SIGNAL(clicked()), SLOT(updateTranslationClicked()));
 
@@ -733,8 +754,8 @@ PreferenceDialog::PreferenceDialog(QWidget* parent)
 
       MuseScore::restoreGeometry(this);
 
-#if !defined(Q_OS_MAC) && !defined(Q_OS_WIN)
-      General->removeTab(General->indexOf(tabUpdate)); // updateTab not needed on Linux
+#if !defined(Q_OS_MAC) && (!defined(Q_OS_WIN) || defined(FOR_WINSTORE))
+      General->removeTab(General->indexOf(tabUpdate)); // updateTab not needed on Linux and not wanted in Windows Store
 #endif
       }
 
@@ -878,6 +899,7 @@ void PreferenceDialog::updateValues()
 
       //Update
       checkUpdateStartup->setChecked(prefs.checkUpdateStartup);
+      checkExtensionsUpdateStartup->setChecked(prefs.checkExtensionsUpdateStartup);
 
       navigatorShow->setChecked(prefs.showNavigator);
       playPanelShow->setChecked(prefs.showPlayPanel);
@@ -902,6 +924,7 @@ void PreferenceDialog::updateValues()
 
       alsaFragments->setValue(prefs.alsaFragments);
       drawAntialiased->setChecked(prefs.antialiasedDrawing);
+      limitScrollArea->setChecked(prefs.limitScrollArea);
       switch(prefs.sessionStart) {
             case SessionStart::EMPTY:  emptySession->setChecked(true); break;
             case SessionStart::LAST:   lastSession->setChecked(true); break;
@@ -972,7 +995,7 @@ void PreferenceDialog::updateValues()
                   connect(portaudioApi, SIGNAL(activated(int)), SLOT(portaudioApiActivated(int)));
 #ifdef USE_PORTMIDI
                   PortMidiDriver* midiDriver = static_cast<PortMidiDriver*>(audio->mididriver());
-                  if(midiDriver){
+                  if (midiDriver) {
                         QStringList midiInputs = midiDriver->deviceInList();
                         int curMidiInIdx = 0;
                         portMidiInput->clear();
@@ -982,6 +1005,19 @@ void PreferenceDialog::updateValues()
                                     curMidiInIdx = i;
                               }
                         portMidiInput->setCurrentIndex(curMidiInIdx);
+
+                        QStringList midiOutputs = midiDriver->deviceOutList();
+                        int curMidiOutIdx = -1; // do not set a midi out device if user never selected one
+                        portMidiOutput->clear();
+                        portMidiOutput->addItem("", -1);
+                        for(int i = 0; i < midiOutputs.size(); ++i) {
+                              portMidiOutput->addItem(midiOutputs.at(i), i);
+                              if (midiOutputs.at(i) == prefs.portMidiOutput)
+                                    curMidiOutIdx = i + 1;
+                              }
+                        portMidiOutput->setCurrentIndex(curMidiOutIdx);
+
+                        portMidiOutputLatencyMilliseconds->setValue(prefs.portMidiOutputLatencyMilliseconds);
                         }
 #endif
                   }
@@ -995,7 +1031,7 @@ void PreferenceDialog::updateValues()
       //
       // score settings
       //
-      scale->setValue(prefs.mag*100.0);
+      scale->setValue(prefs.mag * 100.0);
 
       defaultPlayDuration->setValue(MScore::defaultPlayDuration);
       importStyleFile->setText(prefs.importStyleFile);
@@ -1056,6 +1092,7 @@ void PreferenceDialog::updateValues()
       myTemplates->setText(prefs.myTemplatesPath);
       myPlugins->setText(prefs.myPluginsPath);
       mySoundfonts->setText(prefs.mySoundfontsPath);
+      myExtensions->setText(prefs.myExtensionsPath);
 
       index = exportAudioSampleRate->findData(prefs.exportAudioSampleRate);
       exportAudioSampleRate->setCurrentIndex(index);
@@ -1372,6 +1409,7 @@ void PreferenceDialog::apply()
       prefs.showStartcenter    = showStartcenter->isChecked();
 
       prefs.antialiasedDrawing = drawAntialiased->isChecked();
+      prefs.limitScrollArea    = limitScrollArea->isChecked();
 
       prefs.useJackTransport   = jackDriver->isChecked() && useJackTransport->isChecked();
       prefs.jackTimebaseMaster = becomeTimebaseMaster->isChecked();
@@ -1442,6 +1480,15 @@ void PreferenceDialog::apply()
 
 #ifdef USE_PORTMIDI
       prefs.portMidiInput = portMidiInput->currentText();
+      prefs.portMidiOutput = portMidiOutput->currentText();
+      if (seq->driver() && static_cast<PortMidiDriver*>(static_cast<Portaudio*>(seq->driver())->mididriver())->isSameCoreMidiIacBus(prefs.portMidiInput, prefs.portMidiOutput)) {
+            QMessageBox msgBox;
+            msgBox.setWindowTitle(tr("Possible MIDI Loopback"));
+            msgBox.setIcon(QMessageBox::Warning);
+            msgBox.setText(tr("Warning: You used the same CoreMIDI IAC bus for input and output.  This will cause problematic loopback, whereby MuseScore's outputted MIDI messages will be sent back to MuseScore as input, causing confusion.  To avoid this problem, access Audio MIDI Setup via Spotlight to create a dedicated virtual port for MuseScore's MIDI output, restart MuseScore, return to Preferences, and select your new virtual port for MuseScore's MIDI output.  Other programs may then use that dedicated virtual port to receive MuseScore's MIDI output."));
+            msgBox.exec();
+            }
+      prefs.portMidiOutputLatencyMilliseconds = portMidiOutputLatencyMilliseconds->value();
 #endif
 
       if (lastSession->isChecked())
@@ -1459,6 +1506,7 @@ void PreferenceDialog::apply()
       prefs.myTemplatesPath    = myTemplates->text();
       prefs.myPluginsPath      = myPlugins->text();
       prefs.mySoundfontsPath = mySoundfonts->text();
+      prefs.myExtensionsPath = myExtensions->text();
 
       prefs.exportAudioSampleRate = exportAudioSampleRate->currentData().toInt();
       prefs.exportMp3BitRate   = exportMp3BitRate->currentData().toInt();
@@ -1517,6 +1565,7 @@ void PreferenceDialog::apply()
 
       //update
       prefs.checkUpdateStartup = checkUpdateStartup->isChecked();
+      prefs.checkExtensionsUpdateStartup = checkExtensionsUpdateStartup->isChecked();
 
       prefs.mag         = scale->value()/100.0;
 
@@ -1775,6 +1824,22 @@ void PreferenceDialog::selectImagesDirectory()
             myImages->setText(s);
       }
 
+//---------------------------------------------------------
+//   selectExtensionsDirectory
+//---------------------------------------------------------
+
+void PreferenceDialog::selectExtensionsDirectory()
+      {
+      QString s = QFileDialog::getExistingDirectory(
+         this,
+         tr("Choose Extensions Folder"),
+         myExtensions->text(),
+         QFileDialog::ShowDirsOnly | (preferences.nativeDialogs ? QFileDialog::Options() : QFileDialog::DontUseNativeDialog)
+         );
+      if (!s.isNull())
+            myExtensions->setText(s);
+      }
+
 //---------------------------------------------------------
 //   changeSoundfontPaths
 //---------------------------------------------------------
@@ -1795,6 +1860,7 @@ void PreferenceDialog::changeSoundfontPaths()
 void PreferenceDialog::updateTranslationClicked()
       {
       ResourceManager r(0);
+      r.selectLanguagesTab();
       r.exec();
       }
 
@@ -1943,13 +2009,18 @@ static void updatePluginList(QList<QString>& pluginPathList, const QString& plug
       }
 #endif
 
-void Preferences::updatePluginList()
+void Preferences::updatePluginList(bool forceRefresh)
       {
 #ifdef SCRIPT_INTERFACE
       QList<QString> pluginPathList;
       pluginPathList.append(dataPath + "/plugins");
       pluginPathList.append(mscoreGlobalShare + "plugins");
       pluginPathList.append(myPluginsPath);
+      if (forceRefresh) {
+            pluginList.clear();
+            QQmlEngine* engine=Ms::MScore::qml();
+            engine->clearComponentCache(); //TODO: Check this doesn't have unwanted side effects.
+            }
 
       foreach(QString pluginPath, pluginPathList) {
             Ms::updatePluginList(pluginPathList, pluginPath, pluginList);
